@@ -1,7 +1,7 @@
 /***************************************************************************
  $RCSfile$
                              -------------------
-    cvs         : $Id: askaccount.h 388 2006-03-08 15:38:34Z aquamaniac $
+    cvs         : $Id: askaccount.h 437 2008-03-17 08:12:48Z martin $
     begin       : Mon Mar 01 2004
     copyright   : (C) 2004 by Martin Preuss
     email       : martin@libchipcard.de
@@ -22,13 +22,18 @@ class KBanking;
 #include "askaccount.ui.h"
 #include <qstring.h>
 
+#include <aqbanking/account.h>
+
+
 
 class AskAccount: public AskAccountUi {
   Q_OBJECT
 public:
   AskAccount(KBanking *app,
              const char *bankCode,
-             const char *accountId,
+	     const char *accountId,
+	     const char *accountName,
+             AB_ACCOUNT_TYPE t,
              QWidget* parent=0,
              const char* name=0,
              bool modal=FALSE,
@@ -49,6 +54,8 @@ private:
   AccountListView *_accountList;
   QString _bankCode;
   QString _accountId;
+  QString _accountName;
+  AB_ACCOUNT_TYPE _accountType;
 };
 
 
