@@ -1,7 +1,7 @@
 /***************************************************************************
- $RCSfile: calcpayee.h,v $
+ $RCSfile$
                              -------------------
-    cvs         : $Id: calcpayee.h,v 1.2 2005/03/14 08:40:54 aquamaniac Exp $
+    cvs         : $Id: calcpayee.h 156 2005-03-14 08:40:56Z aquamaniac $
     begin       : Mon Mar 01 2004
     copyright   : (C) 2004 by Martin Preuss
     email       : martin@libchipcard.de
