@@ -1,7 +1,7 @@
 /***************************************************************************
- $RCSfile: editaccount.cpp,v $
+ $RCSfile$
                              -------------------
-    cvs         : $Id: editaccount.cpp,v 1.4 2005/10/02 14:05:27 cstim Exp $
+    cvs         : $Id: editaccount.cpp 438 2008-06-23 17:45:43Z martin $
     begin       : Mon Mar 01 2004
     copyright   : (C) 2004 by Martin Preuss
     email       : martin@libchipcard.de
@@ -36,16 +36,13 @@ EditAccount::EditAccount(KBanking *app,
 ,_account(a)
 ,_isCreate(isCreate) {
 
-  accountNameEdit
-    ->setText(QString::fromUtf8(a->getAccountName().c_str()));
-  accountNumberEdit
-    ->setText(QString::fromUtf8(a->getAccountNumber().c_str()));
-  bankNameEdit
-    ->setText(QString::fromUtf8(a->getBankName().c_str()));
-  bankCodeEdit
-    ->setText(QString::fromUtf8(a->getBankCode().c_str()));
-  ownerEdit
-    ->setText(QString::fromUtf8(a->getOwnerName().c_str()));
+  accountNameEdit->setText(QString::fromUtf8(a->getAccountName().c_str()));
+  accountNumberEdit->setText(QString::fromUtf8(a->getAccountNumber().c_str()));
+  bankNameEdit->setText(QString::fromUtf8(a->getBankName().c_str()));
+  bankCodeEdit->setText(QString::fromUtf8(a->getBankCode().c_str()));
+  ownerEdit->setText(QString::fromUtf8(a->getOwnerName().c_str()));
+  ibanEdit->setText(QString::fromUtf8(a->getIban().c_str()));
+  bicEdit->setText(QString::fromUtf8(a->getBic().c_str()));
 
   updateTransactionsCheck->setChecked(a->getOptionUpdateTransactions());
   updateBalanceCheck->setChecked(a->getOptionUpdateBalance());
@@ -93,6 +90,13 @@ void EditAccount::accept() {
   if (!qs.isEmpty())
     _account->setOwnerName(KBanking::QStringToUtf8String(qs));
 
+  qs=ibanEdit->text();
+  if (!qs.isEmpty())
+    _account->setIban(KBanking::QStringToUtf8String(qs));
+  qs=bicEdit->text();
+  if (!qs.isEmpty())
+    _account->setBic(KBanking::QStringToUtf8String(qs));
+
   _account->setOptionUpdateTransactions(updateTransactionsCheck->isChecked());
   _account->setOptionUpdateBalance(updateBalanceCheck->isChecked());
   _account->setOptionUpdateStandingOrders(updateStandingOrdersCheck->isChecked());
@@ -128,6 +132,9 @@ void EditAccount::slotBankCode() {
     s=AB_BankInfo_GetBankName(bi);
     if (s)
       bankNameEdit->setText(QString::fromUtf8(s));
+    s=AB_BankInfo_GetBic(bi);
+    if (s)
+      bicEdit->setText(QString::fromUtf8(s));
     s=AB_BankInfo_GetLocation(bi);
     AB_BankInfo_free(bi);
   }
