@@ -1,7 +1,7 @@
 /***************************************************************************
  $RCSfile$
                              -------------------
-    cvs         : $Id: account.cpp 393 2006-03-22 16:32:29Z aquamaniac $
+    cvs         : $Id: account.cpp 438 2008-06-23 17:45:43Z martin $
     begin       : Mon Mar 01 2004
     copyright   : (C) 2004 by Martin Preuss
     email       : martin@libchipcard.de
@@ -82,6 +82,12 @@ void Account::updateFromBankingAccount(const AB_ACCOUNT *a) {
   p=AB_Account_GetOwnerName(a);
   if (p)
     _ownerName=p;
+  p=AB_Account_GetIBAN(a);
+  if (p)
+    _iban=p;
+  p=AB_Account_GetBIC(a);
+  if (p)
+    _bic=p;
 }
 
 
@@ -103,6 +109,8 @@ Account::Account(const Account &a)
   _ownerName=a._ownerName;
   _transactions=a._transactions;
   _bankingId=a._bankingId;
+  _iban=a._iban;
+  _bic=a._bic;
   _banking=a._banking;
   _lastTransactionDate=a._lastTransactionDate;
   if (a._accountStatus)
@@ -136,6 +144,8 @@ bool Account::fromDb(GWEN_DB_NODE *db) {
   _bankName=GWEN_DB_GetCharValue(db, "bankName", 0, "");
   _accountName=GWEN_DB_GetCharValue(db, "accountName", 0, "");
   _ownerName=GWEN_DB_GetCharValue(db, "ownerName", 0, "");
+  _iban=GWEN_DB_GetCharValue(db, "iban", 0, "");
+  _bic=GWEN_DB_GetCharValue(db, "bic", 0, "");
 
   dbT=GWEN_DB_GetGroup(db, GWEN_PATH_FLAGS_NAMEMUSTEXIST, "accountStatus");
   if (dbT)
@@ -150,7 +160,12 @@ bool Account::fromDb(GWEN_DB_NODE *db) {
 
     dbTransfer=GWEN_DB_FindFirstGroup(dbT, "transfer");
     while(dbTransfer) {
-      _transfers.push_back(new Transfer(dbTransfer));
+      RefPointer<Transfer> t;
+
+      t=new Transfer(dbTransfer);
+      if (t.ref().getStatus()==Transfer::StatusEnqueued)
+	t.ref().setStatus(Transfer::StatusAborted);
+      _transfers.push_back(t);
       dbTransfer=GWEN_DB_FindNextGroup(dbTransfer, "transfer");
     } // while
   }
@@ -237,6 +252,12 @@ bool Account::toDb(GWEN_DB_NODE *db) {
   if (!_ownerName.empty())
     GWEN_DB_SetCharValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS,
                          "ownerName", _ownerName.c_str());
+  if (!_iban.empty())
+    GWEN_DB_SetCharValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS,
+			 "iban", _iban.c_str());
+  if (!_bic.empty())
+    GWEN_DB_SetCharValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS,
+			 "bic", _bic.c_str());
   GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS,
                       "bankingId", _bankingId);
 
@@ -298,7 +319,7 @@ bool Account::toDb(GWEN_DB_NODE *db) {
     assert(dbTransfer);
     if (!(*itt).ref().toDb(dbTransfer)) {
       DBG_ERROR(0, "Could not save transfer");
-      return AB_ERROR_GENERIC;
+      return GWEN_ERROR_GENERIC;
     }
   }
 
@@ -312,7 +333,7 @@ bool Account::toDb(GWEN_DB_NODE *db) {
     assert(dbTransfer);
     if (!(*its).ref().toDb(dbTransfer)) {
       DBG_ERROR(0, "Could not save transfer");
-      return AB_ERROR_GENERIC;
+      return GWEN_ERROR_GENERIC;
     }
   }
 
@@ -326,7 +347,7 @@ bool Account::toDb(GWEN_DB_NODE *db) {
     assert(dbTransfer);
     if (!(*itt).ref().toDb(dbTransfer)) {
       DBG_ERROR(0, "Could not save dated transfer");
-      return AB_ERROR_GENERIC;
+      return GWEN_ERROR_GENERIC;
     }
   }
 
@@ -407,13 +428,13 @@ void Account::setOwnerName(const std::string &s){
 
 
 
-GWEN_TYPE_UINT32 Account::getBankingId() const{
+uint32_t Account::getBankingId() const{
   return _bankingId;
 }
 
 
 
-void Account::setBankingId(GWEN_TYPE_UINT32 i){
+void Account::setBankingId(uint32_t i){
   _bankingId=i;
 }
 
@@ -448,7 +469,7 @@ bool Account::_loadTransactions(unsigned int fromYear,
                                 unsigned int fromDay) {
   int year, month, day;
   int rv;
-  GWEN_TYPE_UINT32 ymd;
+  uint32_t ymd;
   GWEN_IDLIST *idl;
   AH_STORAGE *st;
 
@@ -469,9 +490,9 @@ bool Account::_loadTransactions(unsigned int fromYear,
   }
   rv=AH_Storage_GetFirstDay(st, idl, &year, &month, &day);
   while(!rv) {
-    GWEN_TYPE_UINT32 id;
+    uint32_t id;
     GWEN_DB_NODE *db;
-    GWEN_TYPE_UINT32 cymd;
+    uint32_t cymd;
 
     DBG_DEBUG(0, "Loading day %04d/%02d/%02d",
               year, month, day);
@@ -529,8 +550,8 @@ bool Account::loadTransactions(GWEN_TIME *fromDate,
                                std::list<Transaction> &tl) {
   int year, month, day;
   int rv;
-  GWEN_TYPE_UINT32 fromYmd;
-  GWEN_TYPE_UINT32 toYmd;
+  uint32_t fromYmd;
+  uint32_t toYmd;
   GWEN_IDLIST *idl;
   AH_STORAGE *st;
 
@@ -571,9 +592,9 @@ bool Account::loadTransactions(GWEN_TIME *fromDate,
   }
   rv=AH_Storage_GetFirstDay(st, idl, &year, &month, &day);
   while(!rv) {
-    GWEN_TYPE_UINT32 id;
+    uint32_t id;
     GWEN_DB_NODE *db;
-    GWEN_TYPE_UINT32 cymd;
+    uint32_t cymd;
 
     cymd=(year<<16)+(month<<8)+day;
     if (cymd>=fromYmd && cymd<=toYmd) {
@@ -631,8 +652,8 @@ bool Account::loadTransactions(GWEN_TIME *fromDate,
                                std::list<RefPointer<Transaction> > &tl) {
   int year, month, day;
   int rv;
-  GWEN_TYPE_UINT32 fromYmd;
-  GWEN_TYPE_UINT32 toYmd;
+  uint32_t fromYmd;
+  uint32_t toYmd;
   GWEN_IDLIST *idl;
   AH_STORAGE *st;
 
@@ -674,9 +695,9 @@ bool Account::loadTransactions(GWEN_TIME *fromDate,
 
   rv=AH_Storage_GetFirstDay(st, idl, &year, &month, &day);
   while(!rv) {
-    GWEN_TYPE_UINT32 id;
+    uint32_t id;
     GWEN_DB_NODE *db;
-    GWEN_TYPE_UINT32 cymd;
+    uint32_t cymd;
 
     cymd=(year<<16)+(month<<8)+day;
     if (cymd>=fromYmd && cymd<=toYmd) {
@@ -799,8 +820,8 @@ const GWEN_TIME *Account::getLastTransactionDate() {
 bool Account::saveTransaction(const Transaction *t) {
   const GWEN_TIME *ti;
   int year, month, day;
-  GWEN_TYPE_UINT32 id;
-  GWEN_TYPE_UINT32 tid;
+  uint32_t id;
+  uint32_t tid;
   GWEN_DB_NODE *dbTransaction;
   GWEN_DB_NODE *dbT;
   AH_STORAGE *st;
@@ -841,7 +862,7 @@ bool Account::saveTransaction(const Transaction *t) {
   assert(tid);
   dbT=AH_Storage_GetFirstTransaction(st, id);
   while(dbT) {
-    GWEN_TYPE_UINT32 i;
+    uint32_t i;
 
     i=GWEN_DB_GetIntValue(dbT, "transactionId", 0, -1);
     if (tid==i)
@@ -879,8 +900,8 @@ bool Account::saveTransaction(const Transaction *t) {
 bool Account::removeTransaction(const Transaction *t) {
   const GWEN_TIME *ti;
   int year, month, day;
-  GWEN_TYPE_UINT32 id;
-  GWEN_TYPE_UINT32 tid;
+  uint32_t id;
+  uint32_t tid;
   GWEN_DB_NODE *dbT;
   AH_STORAGE *st;
 
@@ -911,7 +932,7 @@ bool Account::removeTransaction(const Transaction *t) {
   assert(tid);
   dbT=AH_Storage_GetFirstTransaction(st, id);
   while(dbT) {
-    GWEN_TYPE_UINT32 i;
+    uint32_t i;
 
     i=GWEN_DB_GetIntValue(dbT, "transactionId", 0, -1);
     if (tid==i)
@@ -938,7 +959,7 @@ bool Account::removeTransaction(const Transaction *t) {
 bool Account::addTransaction(RefPointer<Transaction> t) {
   const GWEN_TIME *ti;
   int year, month, day;
-  GWEN_TYPE_UINT32 id;
+  uint32_t id;
   GWEN_DB_NODE *dbT;
   AH_STORAGE *st;
 
@@ -958,7 +979,7 @@ bool Account::addTransaction(RefPointer<Transaction> t) {
     return false;
   }
 
-  DBG_ERROR(0, "Saving transaction for day %04d/%02d/%02d (%s/%s)",
+  DBG_INFO(0, "Saving transaction for day %04d/%02d/%02d (%s/%s)",
             year, month, day,
             _bankCode.c_str(), _accountNumber.c_str());
 
@@ -1120,6 +1141,33 @@ void Account::setOptionExcludeFromGlobalCall(bool b) {
 
 
 
+const std::string &Account::getIban() const {
+  return _iban;
+}
+
+
+
+void Account::setIban(const std::string &s) {
+  _iban=s;
+}
+
+
+
+const std::string &Account::getBic() const {
+  return _bic;
+}
+
+
+
+void Account::setBic(const std::string &s) {
+  _bic=s;
+}
+
+
+
+
+
+
 
 
 
