@@ -1,7 +1,7 @@
 /***************************************************************************
  $RCSfile: app.cpp,v $
                              -------------------
-    cvs         : $Id: app.cpp 396 2006-05-29 17:00:16Z martin $
+    cvs         : $Id: app.cpp 438 2008-06-23 17:45:43Z martin $
     begin       : Mon Mar 01 2004
     copyright   : (C) 2004 by Martin Preuss
     email       : martin@libchipcard.de
@@ -18,11 +18,11 @@
 #include "app.h"
 #include "timport.h"
 
-#include <aqbanking/jobsingletransfer.h>
-
 #include <gwenhywfar/debug.h>
 #include <gwenhywfar/directory.h>
 #include <gwenhywfar/text.h>
+#include <gwenhywfar/io_file.h>
+#include <gwenhywfar/iomanager.h>
 
 #include <stdlib.h>
 #include <sys/types.h>
@@ -85,13 +85,21 @@ int App::init() {
   GWEN_DB_NODE *db;
   GWEN_DB_NODE *dbT;
   int rv;
-  GWEN_TYPE_UINT32 currentVersion;
+  uint32_t currentVersion;
 
   _lastSessionId=0;
 
   rv=FRONTEND_CLASS::init();
-  if (rv)
+  if (rv) {
+    DBG_ERROR(0, "here (%d)", rv);
     return rv;
+  }
+
+  rv=AB_Banking_OnlineInit(getCInterface());
+  if (rv) {
+    DBG_ERROR(0, "here (%d)", rv);
+    return rv;
+  }
 
   /* read configuration */
   db=getAppData();
@@ -158,19 +166,25 @@ int App::init() {
 	if (ba) {
 	  const char *s;
 
-	  DBG_INFO(0, "Updating account");
-	  s=AB_Account_GetBankName(ba);
-	  if (s)
-	    a->setBankName(s);
-	  s=AB_Account_GetAccountName(ba);
-	  if (s)
-	    a->setAccountName(s);
-	  s=AB_Account_GetOwnerName(ba);
-	  if (s)
-	    a->setOwnerName(s);
-        }
+	  DBG_INFO(0, "Possibly updating account");
+	  if (a->getBankName().empty()) {
+	    s=AB_Account_GetBankName(ba);
+	    if (s)
+	      a->setBankName(s);
+	  }
+	  if (a->getAccountName().empty()) {
+	    s=AB_Account_GetAccountName(ba);
+	    if (s)
+	      a->setAccountName(s);
+	  }
+	  if (a->getOwnerName().empty()) {
+	    s=AB_Account_GetOwnerName(ba);
+	    if (s)
+	      a->setOwnerName(s);
+	  }
+	}
         else {
-          DBG_NOTICE(0, "Account "GWEN_TYPE_TMPL_UINT32" no longer "
+          DBG_NOTICE(0, "Account %u no longer "
                      "supported by AqBanking",
                      a->getBankingId());
         }
@@ -184,7 +198,7 @@ int App::init() {
   // read payees
   rv=loadPayees();
   if (rv) {
-    DBG_INFO(0, "Error loading payees (%d)", rv);
+    DBG_ERROR(0, "Error loading payees (%d)", rv);
     return rv;
   }
 
@@ -252,7 +266,6 @@ int App::init() {
         4))) {
     DBG_NOTICE(0, "Updating from old version, scanning all jobs once");
     /* scan all jobs */
-    scanAllJobs();
   }
 
   if ((_lastVersion!=APP_LAST_VERSION_NONE) &&
@@ -271,7 +284,7 @@ int App::init() {
     }
   }
 
-  /* read transfers */
+  /* read transfers (only old transfers, new ones are stored with accounts) */
   dbT=GWEN_DB_GetGroup(db, GWEN_PATH_FLAGS_NAMEMUSTEXIST,
                        "transfers");
   if (dbT) {
@@ -279,14 +292,21 @@ int App::init() {
 
     dbTransfer=GWEN_DB_FindFirstGroup(dbT, "transfer");
     while(dbTransfer) {
-      addTransfer(new Transfer(dbTransfer));
+      RefPointer<Transfer> t;
+
+      t=new Transfer(dbTransfer);
+      if (t.ref().getStatus()==Transfer::StatusEnqueued)
+	t.ref().setStatus(Transfer::StatusAborted);
+      addTransfer(t);
       dbTransfer=GWEN_DB_FindNextGroup(dbTransfer, "transfer");
     } // while
   }
 
   /* sync account list */
-  if (!updateAccountList())
-    return AB_ERROR_GENERIC;
+  if (!updateAccountList()) {
+    DBG_ERROR(0, "Could not update account list (%d)", rv);
+    return GWEN_ERROR_GENERIC;
+  }
   return 0;
 }
 
@@ -363,7 +383,7 @@ int App::fini() {
     assert(dbAccount);
     if (!(*it)->toDb(dbAccount)) {
       DBG_ERROR(0, "Could not save account");
-      return AB_ERROR_GENERIC;
+      return GWEN_ERROR_GENERIC;
     }
   }
 
@@ -394,6 +414,10 @@ int App::fini() {
     delete *cit;
   _categories.clear();
 
+  rv=AB_Banking_OnlineFini(getCInterface());
+  if (rv)
+    return rv;
+
   return FRONTEND_CLASS::fini();
 }
 
@@ -515,7 +539,7 @@ int App::savePayees() {
   if (rv) {
     DBG_INFO(0, "here (%d)", rv);
     GWEN_Buffer_free(bdir);
-    return AB_ERROR_GENERIC;
+    return GWEN_ERROR_GENERIC;
   }
   db=GWEN_DB_Group_new("root");
 
@@ -531,18 +555,18 @@ int App::savePayees() {
       DBG_ERROR(0, "Could not save payee");
       GWEN_DB_Group_free(db);
       GWEN_Buffer_free(bdir);
-      return AB_ERROR_GENERIC;
+      return GWEN_ERROR_GENERIC;
     }
   }
 
   rv=GWEN_DB_WriteFile(db, GWEN_Buffer_GetStart(bdir),
-		       GWEN_DB_FLAGS_DEFAULT);
+		       GWEN_DB_FLAGS_DEFAULT, 0, 2000);
   if (rv) {
     DBG_ERROR(0, "Could not write file \"%s\" (%d)",
 	      GWEN_Buffer_GetStart(bdir), rv);
     GWEN_DB_Group_free(db);
     GWEN_Buffer_free(bdir);
-    return AB_ERROR_GENERIC;
+    return GWEN_ERROR_GENERIC;
   }
 
   GWEN_DB_Group_free(db);
@@ -579,11 +603,12 @@ int App::loadPayees() {
 
   rv=GWEN_DB_ReadFile(db, GWEN_Buffer_GetStart(bdir),
 		      GWEN_DB_FLAGS_DEFAULT |
-		      GWEN_PATH_FLAGS_CREATE_GROUP);
+		      GWEN_PATH_FLAGS_CREATE_GROUP,
+		      0, 2000);
   if (rv) {
     DBG_INFO(0, "Error reading payee file (%d)", rv);
     GWEN_Buffer_free(bdir);
-    return AB_ERROR_GENERIC;
+    return GWEN_ERROR_GENERIC;
   }
 
   dbT=GWEN_DB_GetGroup(db, GWEN_PATH_FLAGS_NAMEMUSTEXIST,
@@ -651,7 +676,7 @@ int App::saveCategories() {
   if (rv) {
     DBG_INFO(0, "here (%d)", rv);
     GWEN_Buffer_free(bdir);
-    return AB_ERROR_GENERIC;
+    return GWEN_ERROR_GENERIC;
   }
   db=GWEN_DB_Group_new("root");
 
@@ -667,13 +692,14 @@ int App::saveCategories() {
   }
 
   rv=GWEN_DB_WriteFile(db, GWEN_Buffer_GetStart(bdir),
-		       GWEN_DB_FLAGS_DEFAULT);
+		       GWEN_DB_FLAGS_DEFAULT,
+		       0, 2000);
   if (rv) {
     DBG_ERROR(0, "Could not write file \"%s\" (%d)",
 	      GWEN_Buffer_GetStart(bdir), rv);
     GWEN_DB_Group_free(db);
     GWEN_Buffer_free(bdir);
-    return AB_ERROR_GENERIC;
+    return GWEN_ERROR_GENERIC;
   }
 
   GWEN_DB_Group_free(db);
@@ -699,10 +725,12 @@ int App::_loadCategoryFile(const char *fname) {
   db=GWEN_DB_Group_new("root");
   rv=GWEN_DB_ReadFile(db, fname,
 		      GWEN_DB_FLAGS_DEFAULT |
-		      GWEN_PATH_FLAGS_CREATE_GROUP);
+		      GWEN_PATH_FLAGS_CREATE_GROUP,
+                      0,
+		      2000);
   if (rv) {
     DBG_INFO(0, "Error reading category file (%d)", rv);
-    return AB_ERROR_GENERIC;
+    return GWEN_ERROR_GENERIC;
   }
 
   dbT=GWEN_DB_GetGroup(db, GWEN_PATH_FLAGS_NAMEMUSTEXIST,
@@ -743,8 +771,8 @@ int App::loadCategories() {
 
 
 
-GWEN_TYPE_UINT64 App::char2uint64(const char *str) {
-  GWEN_TYPE_UINT64 res=0;
+uint64_t App::char2uint64(const char *str) {
+  uint64_t res=0;
   const char *s;
 
   s=str;
@@ -764,7 +792,7 @@ GWEN_TYPE_UINT64 App::char2uint64(const char *str) {
 Account *App::findAccount(const char *bankCode,
                           const char *accountNumber) {
   std::list<Account*>::iterator it;
-  GWEN_TYPE_UINT64 an;
+  uint64_t an;
   Account *a=0;
 
   if (!bankCode)
@@ -790,7 +818,7 @@ Account *App::findAccount(const char *bankCode,
   an=char2uint64(accountNumber);
   if (an) {
     for (it=_accounts.begin(); it!=_accounts.end(); it++) {
-      GWEN_TYPE_UINT64 lan;
+      uint64_t lan;
 
       lan=char2uint64((*it)->getAccountNumber().c_str());
       if (lan) {
@@ -813,6 +841,31 @@ Account *App::findAccount(const char *bankCode,
 
 
 
+Account *App::findAccountByName(const char *accountName) {
+  std::list<Account*>::iterator it;
+  Account *a=0;
+
+  if (!accountName)
+    accountName="*";
+
+  for (it=_accounts.begin(); it!=_accounts.end(); it++) {
+    if (-1!=GWEN_Text_ComparePattern((*it)->getAccountName().c_str(),
+				     accountName, 0)) {
+      if (a) {
+	DBG_ERROR(0, "Ambiguous account specification");
+	return 0;
+      }
+      a=*it;
+    }
+  }
+  if (a)
+    return a;
+
+  return NULL;
+}
+
+
+
 void App::addAccount(Account *a) {
   _accounts.push_back(a);
 }
@@ -959,7 +1012,7 @@ void App::appInfoTransactionsChanged(Account *a){
 
 
 
-GWEN_TYPE_UINT32 App::getNextUniqueId() {
+uint32_t App::getNextUniqueId() {
   return ++_lastUniqueId;
 }
 
@@ -971,14 +1024,14 @@ void App::nextSession() {
 
 
 
-GWEN_TYPE_UINT32 App::getLastSessionId() {
+uint32_t App::getLastSessionId() {
   return _lastSessionId;
 }
 
 
 
 bool App::importContext(AB_IMEXPORTER_CONTEXT *ctx,
-                        GWEN_TYPE_UINT32 flags){
+                        uint32_t flags){
   TransactionImporter imp(this, ctx, flags);
 
   if (!imp.importContext()) {
@@ -991,7 +1044,7 @@ bool App::importContext(AB_IMEXPORTER_CONTEXT *ctx,
 
 
 
-RefPointer<Transfer> App::findTransferByTransactionId(GWEN_TYPE_UINT32 id){
+RefPointer<Transfer> App::findTransferByTransactionId(uint32_t id){
   std::list<Account*>::iterator it;
 
   for (it=_accounts.begin(); it!=_accounts.end(); it++) {
@@ -1009,159 +1062,20 @@ RefPointer<Transfer> App::findTransferByTransactionId(GWEN_TYPE_UINT32 id){
 
 
 
-void App::scanJobList(AB_JOB_LIST2 *jl) {
-  AB_JOB_LIST2_ITERATOR *it;
-  AB_JOB *j;
+RefPointer<Transfer> App::findTransferByJobId(uint32_t id){
+  std::list<Account*>::iterator it;
 
-  it=AB_Job_List2_First(jl);
-  if (!it)
-    return;
+  for (it=_accounts.begin(); it!=_accounts.end(); it++) {
+    std::list<RefPointer<Transfer> >::iterator itt;
 
-  j=AB_Job_List2Iterator_Data(it);
-  assert(j);
-  while(j) {
-    switch(AB_Job_GetType(j)) {
-    case AB_Job_TypeGetBalance:
-      break;
-    case AB_Job_TypeGetTransactions:
-      break;
-    case AB_Job_TypeTransfer: {
-      const AB_TRANSACTION *bt;
+    for (itt=(*it)->getTransfers().begin();
+         itt!=(*it)->getTransfers().end();
+         itt++)
+      if (id==(*itt).ref().getJobId())
+        return *itt;
+  } // for accounts
 
-      bt=AB_JobSingleTransfer_GetTransaction(j);
-      if (bt) {
-        GWEN_TYPE_UINT32 tid;
-        RefPointer<Transfer> t;
-
-        DBG_INFO(0, "Checking transfer");
-        tid=AB_Transaction_GetUniqueId(bt);
-        if (tid) {
-          t=findTransferByTransactionId(tid);
-          if (t.isValid()) {
-            DBG_INFO(0, "Transfer is known");
-            if (t.ref().getTransactionId()==0)
-              t.ref().setTransactionId(getNextUniqueId());
-          }
-          if (!t.isValid()) {
-            t=new Transfer(bt);
-            t.ref().setJobId(AB_Job_GetJobId(j));
-            DBG_INFO(0, "Adding transfer");
-            addTransfer(t);
-          }
-          if (!t.ref().getDate()) {
-            DBG_INFO(0, "Adjusting date");
-            t.ref().setDate(AB_Job_GetLastStatusChange(j));
-          }
-          if (t.ref().getStatus()<Transfer::StatusOk) {
-            const GWEN_STRINGLIST *sl;
-
-            sl=AB_Transaction_GetRemoteName(bt);
-            if (sl) {
-              GWEN_STRINGLISTENTRY *se;
-
-              t.ref().clearRemoteName();
-              se=GWEN_StringList_FirstEntry(sl);
-              while(se) {
-                const char *p;
-
-                p=GWEN_StringListEntry_Data(se);
-                assert(p);
-                t.ref().addRemoteName(p);
-                se=GWEN_StringListEntry_Next(se);
-              } /* while */
-            }
-            sl=AB_Transaction_GetPurpose(bt);
-            if (sl) {
-              GWEN_STRINGLISTENTRY *se;
-
-              t.ref().clearPurpose();
-              se=GWEN_StringList_FirstEntry(sl);
-              while(se) {
-                const char *p;
-
-                p=GWEN_StringListEntry_Data(se);
-                assert(p);
-                t.ref().addPurpose(p);
-                se=GWEN_StringListEntry_Next(se);
-              } /* while */
-            }
-
-            switch(AB_Job_GetStatus(j)) {
-            case AB_Job_StatusEnqueued:
-              DBG_INFO(0, "Status: Enqueued");
-              t.ref().setStatus(Transfer::StatusEnqueued);
-              break;
-
-            case AB_Job_StatusNew:
-            case AB_Job_StatusUpdated:
-            case AB_Job_StatusSent:
-            case AB_Job_StatusPending:
-              DBG_INFO(0, "Status: Pending");
-              t.ref().setStatus(Transfer::StatusPending);
-              break;
-            case AB_Job_StatusFinished:
-              DBG_INFO(0, "Status: Finished");
-              t.ref().setStatus(Transfer::StatusOk);
-              break;
-            case AB_Job_StatusError:
-              DBG_INFO(0, "Status: Error");
-              t.ref().setStatus(Transfer::StatusError);
-              break;
-            default:
-              t.ref().setStatus(Transfer::StatusUnknown);
-              DBG_INFO(0, "Transfer status unknown");
-              break;
-            } // switch
-          } // if status is changeable
-          else {
-            DBG_INFO(0, "Unmutable status %d", t.ref().getStatus());
-          }
-        } // if transaction id is valid
-        else {
-          DBG_INFO(0, "Ignoring job with no transfer id");
-        }
-      }
-      break;
-    }
-    case AB_Job_TypeDebitNote:
-      break;
-    default:
-      break;
-    } // switch
-
-    j=AB_Job_List2Iterator_Next(it);
-  } // while
-  AB_Job_List2Iterator_free(it);
-}
-
-
-
-void App::scanAllJobs() {
-  AB_JOB_LIST2 *jl;
-
-  jl=AB_Banking_GetPendingJobs(getCInterface());
-  if (jl) {
-    scanJobList(jl);
-    AB_Job_List2_FreeAll(jl);
-  }
-
-  jl=AB_Banking_GetArchivedJobs(getCInterface());
-  if (jl) {
-    scanJobList(jl);
-    AB_Job_List2_FreeAll(jl);
-  }
-
-  jl=AB_Banking_GetFinishedJobs(getCInterface());
-  if (jl) {
-    scanJobList(jl);
-    AB_Job_List2_FreeAll(jl);
-  }
-
-  jl=AB_Banking_GetEnqueuedJobs(getCInterface());
-  if (jl) {
-    scanJobList(jl);
-    AB_Job_List2_free(jl);
-  }
+  return 0;
 }
 
 
@@ -1373,7 +1287,7 @@ bool App::_ruleAppliesToTransaction(GWEN_DB_NODE *dbRule,
       const GWEN_TIME *ti;
       int year, month, day;
       int tyear, tmonth, tday;
-      GWEN_TYPE_UINT32 secs;
+      uint32_t secs;
       const char *s;
   
       cti=GWEN_CurrentTime();
@@ -1578,7 +1492,7 @@ bool App::_ruleAppliesToTransaction(GWEN_DB_NODE *dbRule,
         assert(s);
         tv1=AB_Value_fromString(s);
         assert(tv1);
-        xaMatch=AB_Value_IsEqual(v, tv1);
+	xaMatch=(AB_Value_Compare(v, tv1)==0);
         AB_Value_free(tv1);
         break;
       case TMatcherAmountRange_Range:
@@ -1735,7 +1649,7 @@ bool App::getMatchingTransactions(GWEN_DB_NODE *dbRule,
     GWEN_TIME *tti;
     int year, month, day;
     int tyear, tmonth, tday;
-    GWEN_TYPE_UINT32 secs;
+    uint32_t secs;
     const char *s;
 
     cti=GWEN_CurrentTime();
@@ -1925,17 +1839,15 @@ int App::exportTransactions(GWEN_DB_NODE *dbRule,
   const char *exporterName;
   GWEN_DB_NODE *dbProfile;
   int rv;
-  GWEN_BUFFEREDIO *bio;
+  GWEN_IO_LAYER *io;
   const char *fname;
   int fd;
-  GWEN_ERRORCODE err;
-  GWEN_TYPE_UINT32 wid;
 
   /* get exporter name */
   exporterName=GWEN_DB_GetCharValue(dbRule, "exporterText", 0, 0);
   if (!exporterName) {
     DBG_ERROR(0, "No exporter set");
-    return AB_ERROR_INVALID;
+    return GWEN_ERROR_INVALID;
   }
 
   /* get profile */
@@ -1943,21 +1855,21 @@ int App::exportTransactions(GWEN_DB_NODE *dbRule,
                              "profile");
   if (!dbProfile) {
     DBG_ERROR(0, "No export profile set");
-    return AB_ERROR_INVALID;
+    return GWEN_ERROR_INVALID;
   }
 
   /* get exporter */
   exporter=AB_Banking_GetImExporter(getCInterface(), exporterName);
   if (!exporter) {
     DBG_ERROR(0, "Exporter %s not found", exporterName);
-    return AB_ERROR_INVALID;
+    return GWEN_ERROR_INVALID;
   }
 
   /* check for file */
   fname=GWEN_DB_GetCharValue(dbRule, "outfileText", 0, 0);
   if (!fname) {
     DBG_ERROR(0, "No file name set");
-    return AB_ERROR_INVALID;
+    return GWEN_ERROR_INVALID;
   }
 
   /* open file */
@@ -1971,10 +1883,17 @@ int App::exportTransactions(GWEN_DB_NODE *dbRule,
 		  );
   if (fd==-1) {
     DBG_ERROR(0, "open(%s): %s", fname, strerror(errno));
-    return AB_ERROR_GENERIC;
+    return GWEN_ERROR_GENERIC;
+  }
+
+  io=GWEN_Io_LayerFile_new(0, fd);
+  GWEN_Io_Layer_AddFlags(io, GWEN_IO_LAYER_FLAGS_DONTCLOSE);
+  rv=GWEN_Io_Manager_RegisterLayer(io);
+  if (rv<0) {
+    DBG_ERROR(0, "Error registering layer (%d)", rv);
+    close(fd);
+    return rv;
   }
-  bio=GWEN_BufferedIO_File_new(fd);
-  GWEN_BufferedIO_SetWriteBuffer(bio, 0, 1024);
 
   /* add transactions to context */
   ctx=AB_ImExporterContext_new();
@@ -1984,45 +1903,40 @@ int App::exportTransactions(GWEN_DB_NODE *dbRule,
     bt=(*it).ref().toBankingTransaction();
     assert(bt);
     AB_ImExporterContext_AddTransaction(ctx, bt);
-  } // for
+  } /* for */
+
+  AB_Banking_FillGapsInImExporterContext(getCInterface(), ctx);
 
   /* let exporter work */
-  wid=AB_Banking_ProgressStart(getCInterface(),
-                               "Exporting Transactions ...",
-                               "The selected transactions are now being "
-                               "exported, please wait...",
-                               tl.size());
-  rv=AB_ImExporter_Export(exporter, ctx, bio, dbProfile);
+  rv=AB_ImExporter_Export(exporter,
+			  ctx,
+			  io,
+			  dbProfile,
+			  0);
   AB_ImExporterContext_free(ctx);
-  if (rv) {
+  if (rv<0) {
     DBG_ERROR(0, "Export plugin signalled an error (%d)", rv);
-    AB_Banking_ProgressLog(getCInterface(),
-                           wid,
-                           AB_Banking_LogLevelError,
-                           "Export plugin signalled an error");
-    GWEN_BufferedIO_Abandon(bio);
-    GWEN_BufferedIO_free(bio);
+    GWEN_Io_Layer_free(io);
+    close(fd);
+    return rv;
   }
   else {
-    err=GWEN_BufferedIO_Close(bio);
-    if (!GWEN_Error_IsOk(err)) {
-      DBG_ERROR_ERR(0, err);
-      AB_Banking_ProgressLog(getCInterface(),
-                             wid,
-                             AB_Banking_LogLevelError,
-                             "Error closing output stream");
-      GWEN_BufferedIO_Abandon(bio);
-      GWEN_BufferedIO_free(bio);
-      return AB_ERROR_GENERIC;
+    /* flush */
+    rv=GWEN_Io_Layer_DisconnectRecursively(io, NULL, 0, 0, 2000);
+    if (rv<0) {
+      DBG_ERROR(0, "Error on disconnect (%d)", rv);
+      GWEN_Io_Layer_free(io);
+      close(fd);
+      return rv;
+    }
+    GWEN_Io_Layer_free(io);
+
+    if (close(fd)) {
+      DBG_ERROR(0, "close(): %s", strerror(errno));
+      return GWEN_ERROR_IO;
     }
-    AB_Banking_ProgressLog(getCInterface(),
-                           wid,
-                           AB_Banking_LogLevelNotice,
-                           "Export completed.");
-    GWEN_BufferedIO_free(bio);
   }
-  AB_Banking_ProgressEnd(getCInterface(), wid);
-  return rv;
+  return 0;
 }
 
 
@@ -2197,31 +2111,31 @@ GWEN_TIME *App::adjustedDate(const GWEN_TIME *pt){
 
 
 
-GWEN_TYPE_UINT32 App::getLastVersion() const {
+uint32_t App::getLastVersion() const {
   return _lastVersion;
 }
 
 
 
-GWEN_TYPE_UINT32 App::getFuzzyThreshold() const {
+uint32_t App::getFuzzyThreshold() const {
   return _fuzzyThreshold;
 }
 
 
 
-void App::setFuzzyThreshold(GWEN_TYPE_UINT32 i) {
+void App::setFuzzyThreshold(uint32_t i) {
   _fuzzyThreshold=i;
 }
 
 
 
-GWEN_TYPE_UINT32 App::getSensitiveFuzzyThreshold() const {
+uint32_t App::getSensitiveFuzzyThreshold() const {
   return _sensitiveFuzzyThreshold;
 }
 
 
 
-void App::setSensitiveFuzzyThreshold(GWEN_TYPE_UINT32 i) {
+void App::setSensitiveFuzzyThreshold(uint32_t i) {
   _sensitiveFuzzyThreshold=i;
 }
 
@@ -2533,8 +2447,8 @@ std::string App::tr(const char *s) {
  * Gtk-Gnutella is licensed under the GPL.
  */
 
-GWEN_TYPE_UINT32 App__word_similarity(const char *a, const char *b){
-  GWEN_TYPE_UINT32 score = 0;
+uint32_t App__word_similarity(const char *a, const char *b){
+  uint32_t score = 0;
   size_t l = 0;
 
   while (*a && *b) {
@@ -2606,10 +2520,10 @@ GWEN_STRINGLIST *App__make_word_list(const char *s) {
 
 
 
-GWEN_TYPE_UINT32 App__cmp_word_list(const char *s,
+uint32_t App__cmp_word_list(const char *s,
                                     GWEN_STRINGLIST *words) {
-  GWEN_TYPE_UINT32 score = 0;
-  GWEN_TYPE_UINT32 n = 0;
+  uint32_t score = 0;
+  uint32_t n = 0;
   GWEN_STRINGLISTENTRY *se;
 
   se=GWEN_StringList_FirstEntry(words);
@@ -2632,10 +2546,10 @@ GWEN_TYPE_UINT32 App__cmp_word_list(const char *s,
 
 
 
-GWEN_TYPE_UINT32 App__find_score(GWEN_STRINGLIST *a,
+uint32_t App__find_score(GWEN_STRINGLIST *a,
 				       GWEN_STRINGLIST *b) {
-  GWEN_TYPE_UINT32 score = 0;
-  GWEN_TYPE_UINT32 n = 0;
+  uint32_t score = 0;
+  uint32_t n = 0;
   GWEN_STRINGLISTENTRY *se;
 
   se=GWEN_StringList_FirstEntry(a);
@@ -2677,7 +2591,12 @@ int App__convertFromUtf8(const char *text,
   else {
     char *outbuf;
     char *pOutbuf;
+#if defined(OS_LINUX) | defined(OS_WIN32)
     char *pInbuf;
+#else
+    /* For Mac OSX */
+    const char *pInbuf;
+#endif
     size_t inLeft;
     size_t outLeft;
     size_t done;
@@ -2722,10 +2641,10 @@ int App__convertFromUtf8(const char *text,
 
 
 
-GWEN_TYPE_UINT32 App_FuzzyCompare(const char *str1, const char *str2) {
+uint32_t App_FuzzyCompare(const char *str1, const char *str2) {
   GWEN_STRINGLIST *a;
   GWEN_STRINGLIST *b;
-  GWEN_TYPE_UINT32 score;
+  uint32_t score;
   unsigned int l1, l2;
   GWEN_BUFFER *buf1;
   GWEN_BUFFER *buf2;
