@@ -1,7 +1,7 @@
 /***************************************************************************
  $RCSfile: timport.cpp,v $
                              -------------------
-    cvs         : $Id: timport.cpp 396 2006-05-29 17:00:16Z martin $
+    cvs         : $Id: timport.cpp 437 2008-03-17 08:12:48Z martin $
     begin       : Mon Mar 01 2004
     copyright   : (C) 2004 by Martin Preuss
     email       : martin@libchipcard.de
@@ -20,7 +20,7 @@
 #include "standingorder.h"
 
 #include <gwenhywfar/debug.h>
-#include <gwenhywfar/waitcallback.h>
+#include <gwenhywfar/gui.h>
 
 
 
@@ -111,6 +111,12 @@ const std::string &TransactionImporter::AccountData::bankCode(){
 
 
 
+const std::string &TransactionImporter::AccountData::accountName(){
+  return _account->getAccountName();
+}
+
+
+
 TransactionImporter::DayData*
 TransactionImporter::AccountData::getNewDayData(int year,
                                                 int month,
@@ -136,7 +142,7 @@ TransactionImporter::AccountData::getNewDayData(int year,
 
 TransactionImporter::TransactionImporter(App *kb,
 					 AB_IMEXPORTER_CONTEXT *ctx,
-                                         GWEN_TYPE_UINT32 flags)
+                                         uint32_t flags)
 :_app(kb)
 ,_context(ctx)
 ,_flags(flags){
@@ -167,7 +173,9 @@ TransactionImporter::~TransactionImporter(){
 
 Account*
 TransactionImporter::askUserForAccount(const std::string &bankCode,
-                                       const std::string &accountNumber){
+				       const std::string &accountNumber,
+				       const std::string &accountName,
+				       AB_ACCOUNT_TYPE t){
   return 0;
 }
 
@@ -221,69 +229,6 @@ TransactionImporter::presentTransactionsToUser(std::list<RefPointer<Transaction>
 
 
 
-bool TransactionImporter::_collapseSplits(AB_TRANSACTION *t,
-                                          const std::string &bankCode,
-                                          const std::string &accountId){
-  int n;
-
-  n=AB_Split_List_GetCount(AB_Transaction_GetSplits(t));
-  if (n) {
-    AB_SPLIT *sp;
-    const char *s;
-
-    if (n>2) {
-      DBG_ERROR(0,
-		"Transfer contains more than two splits, unable to handle");
-      return false;
-    }
-
-    /* find split of this account */
-    sp=AB_Split_List_First(AB_Transaction_GetSplits(t));
-    while(sp) {
-      const char *spBankCode;
-      const char *spAccountId;
-
-      spBankCode=AB_Split_GetBankCode(sp);
-      spAccountId=AB_Split_GetAccountNumber(sp);
-      if (spBankCode && spAccountId) {
-	if (strcasecmp(spBankCode, bankCode.c_str())==0 &&
-	    strcasecmp(spAccountId, accountId.c_str())==0)
-          /* split found */
-	  break;
-      }
-      sp=AB_Split_List_Next(sp);
-    }
-    if (sp) {
-      /* read data from local split */
-      AB_Transaction_SetLocalAccountNumber(t, AB_Split_GetAccountNumber(sp));
-      AB_Transaction_SetLocalBankCode(t, AB_Split_GetBankCode(sp));
-      s=GWEN_StringList_FirstString(AB_Split_GetName(sp));
-      AB_Transaction_SetLocalName(t, s);
-      AB_Transaction_SetValue(t, AB_Split_GetValue(sp));
-
-      /* now get remote split */
-      sp=AB_Split_List_Next(sp);
-      if (!sp)
-        sp=AB_Split_List_First(AB_Transaction_GetSplits(t));
-    }
-    else
-      sp=AB_Split_List_First(AB_Transaction_GetSplits(t));
-
-    /* read remote data */
-    AB_Transaction_SetRemoteAccountNumber(t, AB_Split_GetAccountNumber(sp));
-    AB_Transaction_SetRemoteBankCode(t, AB_Split_GetBankCode(sp));
-    AB_Transaction_SetPurpose(t, AB_Split_GetPurpose(sp));
-    AB_Transaction_SetRemoteName(t, AB_Split_GetName(sp));
-
-    /* remove splits */
-    AB_Split_List_Clear(AB_Transaction_GetSplits(t));
-  }
-
-  return true;
-}
-
-
-
 bool TransactionImporter::sortNewTransactions() {
   AB_IMEXPORTER_ACCOUNTINFO *ai;
 
@@ -303,7 +248,9 @@ bool TransactionImporter::sortNewTransactions() {
               AB_ImExporterAccountInfo_GetAccountNumber(ai));
     currentAccount=
       getAccountData(AB_ImExporterAccountInfo_GetBankCode(ai),
-                     AB_ImExporterAccountInfo_GetAccountNumber(ai));
+		     AB_ImExporterAccountInfo_GetAccountNumber(ai),
+		     AB_ImExporterAccountInfo_GetAccountName(ai),
+		     AB_ImExporterAccountInfo_GetType(ai));
     if (!currentAccount) {
       // TODO: ask user whether he wants to abort
       return false;
@@ -317,21 +264,7 @@ bool TransactionImporter::sortNewTransactions() {
       const GWEN_TIME *ti;
       int d, month, year;
 
-      tr=0;
-      if (AB_Split_List_GetCount(AB_Transaction_GetSplits(t))) {
-        AB_TRANSACTION *nt;
-
-        nt=AB_Transaction_dup(t);
-        if (!_collapseSplits(nt,
-                             currentAccount->bankCode(),
-                             currentAccount->accountNumber())) {
-          DBG_ERROR(0, "Error collapsing splits of a transaction");
-        }
-        tr=new Transaction(nt);
-        AB_Transaction_free(nt);
-      }
-      else
-        tr=new Transaction(t);
+      tr=new Transaction(t);
 
       if (tr) {
         tr->gatherInfoFromPurpose();
@@ -491,7 +424,7 @@ TransactionImporter::_checkDay(Account *a,
        it!=d->getTransactions()->end();
        it++) {
     const GWEN_TIME *ti;
-    GWEN_TYPE_UINT32 id;
+    uint32_t id;
 
     // assign unique id if there was none
     id=(*it).ref().getTransactionId();
@@ -522,7 +455,7 @@ TransactionImporter::_checkDay(Account *a,
 
   if (!(_flags & TRANSACTION_IMPORTER_FLAGS_OVERWRITE_DAYS)) {
     AH_STORAGE *st;
-    GWEN_TYPE_UINT32 hdl;
+    uint32_t hdl;
   
     st=a->getStorage();
     assert(st);
@@ -660,17 +593,21 @@ bool TransactionImporter::_assignCatsAndPayees() {
   bool bAskUserForPayee;
   bool bAskUserForCategory;
   unsigned int cnt;
+  uint32_t pid;
 
   bAskUserForPayee=_app->optionAutoAskForPayee();
   bAskUserForCategory=_app->optionAutoAskForCategory();
 
-  GWEN_WaitCallback_EnterWithText(GWEN_WAITCALLBACK_ID_FAST,
-				  _app->tr("Assigning categories and payees").c_str(),
-				  _app->tr("transaction(s)").c_str(),
-				  0);
-  GWEN_WaitCallback_SetProgressTotal(_toAdd.size());
+
+  pid=GWEN_Gui_ProgressStart(GWEN_GUI_PROGRESS_DELAY |
+			     GWEN_GUI_PROGRESS_ALLOW_EMBED |
+			     GWEN_GUI_PROGRESS_SHOW_PROGRESS |
+			     GWEN_GUI_PROGRESS_SHOW_ABORT,
+			     _app->tr("Assigning categories and payees").c_str(),
+			     _app->tr("transaction(s)").c_str(),
+			     _toAdd.size(),
+			     0);
   cnt=0;
-  GWEN_WaitCallback_SetProgressPos(cnt);
 
   /* assign payees and categories */
   for (it=_toAdd.begin(); it!=_toAdd.end(); it++) {
@@ -678,7 +615,7 @@ bool TransactionImporter::_assignCatsAndPayees() {
   
     if ((*it).ref().getPayee().empty()) {
       Payee *payee=0;
-  
+
       if (_app->optionAutoAssignPayee()) {
 	payee=_app->findPayeeByTransactionMatch((*it).ptr());
 	if (!payee) {
@@ -689,7 +626,7 @@ bool TransactionImporter::_assignCatsAndPayees() {
 	    payee=askUserForPayee((*it), aborted, askNoMore);
 	    if (!payee && aborted) {
 	      DBG_ERROR(0, "User aborted import");
-	      GWEN_WaitCallback_Leave();
+	      GWEN_Gui_ProgressEnd(pid);
 	      return false;
 	    }
 	    if (askNoMore)
@@ -717,7 +654,7 @@ bool TransactionImporter::_assignCatsAndPayees() {
 	    category=askUserForCategory((*it), aborted, askNoMore);
 	    if (!category && aborted) {
 	      DBG_ERROR(0, "User aborted import");
-	      GWEN_WaitCallback_Leave();
+	      GWEN_Gui_ProgressEnd(pid);
 	      return false;
 	    }
 	    if (askNoMore)
@@ -733,9 +670,12 @@ bool TransactionImporter::_assignCatsAndPayees() {
     }
   
     cnt++;
-    GWEN_WaitCallbackProgress(cnt);
+    if (GWEN_ERROR_USER_ABORTED==GWEN_Gui_ProgressAdvance(pid, cnt)) {
+      GWEN_Gui_ProgressEnd(pid);
+      return false;
+    }
   } /* for */
-  GWEN_WaitCallback_Leave();
+  GWEN_Gui_ProgressEnd(pid);
   return true;
 }
 
@@ -746,14 +686,17 @@ TransactionImporter::_adjustPCValues(std::list<RefPointer<Transaction> > &tl,
 				     bool add){
   std::list<RefPointer<Transaction> >::iterator it;
   unsigned int cnt;
+  uint32_t pid;
 
-  GWEN_WaitCallback_EnterWithText(GWEN_WAITCALLBACK_ID_FAST,
-				  _app->tr("Adjusting amounts of categories and payees").c_str(),
-				  _app->tr("transaction(s)").c_str(),
-				  0);
-  GWEN_WaitCallback_SetProgressTotal(tl.size());
+  pid=GWEN_Gui_ProgressStart(GWEN_GUI_PROGRESS_DELAY |
+			     GWEN_GUI_PROGRESS_ALLOW_EMBED |
+			     GWEN_GUI_PROGRESS_SHOW_PROGRESS |
+			     GWEN_GUI_PROGRESS_SHOW_ABORT,
+			     _app->tr("Adjusting amounts of categories and payees").c_str(),
+			     _app->tr("transaction(s)").c_str(),
+			     tl.size(),
+			     0);
   cnt=0;
-  GWEN_WaitCallback_SetProgressPos(cnt);
   for (it=tl.begin(); it!=tl.end(); it++) {
     const AB_VALUE *v;
 
@@ -819,9 +762,13 @@ TransactionImporter::_adjustPCValues(std::list<RefPointer<Transaction> > &tl,
       }
     } // if v
     cnt++;
-    GWEN_WaitCallbackProgress(cnt);
+
+    if (GWEN_ERROR_USER_ABORTED==GWEN_Gui_ProgressAdvance(pid, cnt)) {
+      GWEN_Gui_ProgressEnd(pid);
+      return false;
+    }
   } // for
-  GWEN_WaitCallback_Leave();
+  GWEN_Gui_ProgressEnd(pid);
 
   return true;
 }
@@ -831,6 +778,7 @@ TransactionImporter::_adjustPCValues(std::list<RefPointer<Transaction> > &tl,
 bool TransactionImporter::importTransactions(){
   std::list<RefPointer<Transaction> >::iterator it;
   unsigned int cnt;
+  uint32_t pid;
 
   if (!_checkTransactions())
     return false;
@@ -849,13 +797,15 @@ bool TransactionImporter::importTransactions(){
     return false;
 
   /* add new transactions */
-  GWEN_WaitCallback_EnterWithText(GWEN_WAITCALLBACK_ID_FAST,
-				  _app->tr("Adding transactions to accounts").c_str(),
-				  _app->tr("transaction(s)").c_str(),
-				  0);
-  GWEN_WaitCallback_SetProgressTotal(_toAdd.size());
+  pid=GWEN_Gui_ProgressStart(GWEN_GUI_PROGRESS_DELAY |
+			     GWEN_GUI_PROGRESS_ALLOW_EMBED |
+			     GWEN_GUI_PROGRESS_SHOW_PROGRESS |
+			     GWEN_GUI_PROGRESS_SHOW_ABORT,
+			     _app->tr("Adding transactions to accounts").c_str(),
+			     _app->tr("transaction(s)").c_str(),
+			     _toAdd.size(),
+			     0);
   cnt=0;
-  GWEN_WaitCallback_SetProgressPos(cnt);
   for (it=_toAdd.begin(); it!=_toAdd.end(); it++) {
     Account *a;
 
@@ -864,19 +814,24 @@ bool TransactionImporter::importTransactions(){
     assert(a);
     a->addTransaction(*it);
     cnt++;
-    GWEN_WaitCallbackProgress(cnt);
+    if (GWEN_ERROR_USER_ABORTED==GWEN_Gui_ProgressAdvance(pid, cnt)) {
+      GWEN_Gui_ProgressEnd(pid);
+      return false;
+    }
   }
-  GWEN_WaitCallback_Leave();
+  GWEN_Gui_ProgressEnd(pid);
 
   /* del old transactions */
   if (_toDel.size()) {
-    GWEN_WaitCallback_EnterWithText(GWEN_WAITCALLBACK_ID_FAST,
-				    _app->tr("Deleting transactions from accounts").c_str(),
-				    _app->tr("transaction(s)").c_str(),
-				    0);
-    GWEN_WaitCallback_SetProgressTotal(_toDel.size());
+    pid=GWEN_Gui_ProgressStart(GWEN_GUI_PROGRESS_DELAY |
+			       GWEN_GUI_PROGRESS_ALLOW_EMBED |
+			       GWEN_GUI_PROGRESS_SHOW_PROGRESS |
+			       GWEN_GUI_PROGRESS_SHOW_ABORT,
+			       _app->tr("Deleting transactions from accounts").c_str(),
+			       _app->tr("transaction(s)").c_str(),
+			       _toDel.size(),
+			       0);
     cnt=0;
-    GWEN_WaitCallback_SetProgressPos(cnt);
     for (it=_toDel.begin(); it!=_toDel.end(); it++) {
       Account *a;
   
@@ -885,9 +840,12 @@ bool TransactionImporter::importTransactions(){
       assert(a);
       a->removeTransaction((*it).ptr());
       cnt++;
-      GWEN_WaitCallbackProgress(cnt);
+      if (GWEN_ERROR_USER_ABORTED==GWEN_Gui_ProgressAdvance(pid, cnt)) {
+	GWEN_Gui_ProgressEnd(pid);
+	return false;
+      }
     }
-    GWEN_WaitCallback_Leave();
+    GWEN_Gui_ProgressEnd(pid);
   }
 
   return true;
@@ -944,40 +902,90 @@ bool TransactionImporter::importAccounts(){
 
   ai=AB_ImExporterContext_GetFirstAccountInfo(_context);
   while(ai) {
-    Account *a;
+    TransactionImporter::AccountData *ad;
 
-    DBG_DEBUG(0, "Checking account (%s/%s)",
-               AB_ImExporterAccountInfo_GetBankCode(ai),
-               AB_ImExporterAccountInfo_GetAccountNumber(ai));
-    a=_app->findAccount(AB_ImExporterAccountInfo_GetBankCode(ai),
-                        AB_ImExporterAccountInfo_GetAccountNumber(ai));
-    if (!a) {
+    ad=getAccountData(AB_ImExporterAccountInfo_GetBankCode(ai),
+		      AB_ImExporterAccountInfo_GetAccountNumber(ai),
+		      AB_ImExporterAccountInfo_GetAccountName(ai),
+		      AB_ImExporterAccountInfo_GetType(ai));
+    if (!ad) {
       /* account not found, TODO: ask the user to add it */
       DBG_ERROR(0, "Account \"%s/%s\" not found",
-                AB_ImExporterAccountInfo_GetBankCode(ai),
-                AB_ImExporterAccountInfo_GetAccountNumber(ai));
+		AB_ImExporterAccountInfo_GetBankCode(ai),
+		AB_ImExporterAccountInfo_GetAccountNumber(ai));
     }
     else {
+      Account *a;
       AB_ACCOUNT_STATUS *bestBooked;
       AB_ACCOUNT_STATUS *bestNoted;
 
-      /* get best booked balance */
+      a=ad->account();
+      assert(a);
+
+      /* combine best booked and best noted balances */
       bestBooked=_getBestBalance(ai, false);
       bestNoted=_getBestBalance(ai, true);
 
-      if (bestBooked) {
-        if (AB_AccountStatus_GetNotedBalance(bestBooked)==0) {
-	  if (bestNoted) {
-	    const AB_BALANCE *bestNotedBalance;
+      if (bestBooked || bestNoted) {
+	AB_ACCOUNT_STATUS *finalStatus;
 
-	    bestNotedBalance=AB_AccountStatus_GetNotedBalance(bestNoted);
-	    AB_AccountStatus_SetNotedBalance(bestBooked, bestNotedBalance);
+	finalStatus=AB_AccountStatus_new();
+
+	if (bestBooked) {
+	  const AB_BALANCE *bal;
+
+	  bal=AB_AccountStatus_GetBookedBalance(bestBooked);
+	  if (bal)
+	    AB_AccountStatus_SetBookedBalance(finalStatus, bal);
+	  if (AB_AccountStatus_GetBankLine(finalStatus)==NULL) {
+	    const AB_VALUE *v=AB_AccountStatus_GetBankLine(bestBooked);
+	    AB_AccountStatus_SetBankLine(finalStatus, v);
 	  }
+	  if (AB_AccountStatus_GetDisposable(finalStatus)==NULL) {
+	    const AB_VALUE *v=AB_AccountStatus_GetDisposable(bestBooked);
+	    AB_AccountStatus_SetDisposable(finalStatus, v);
+	  }
+	  if (AB_AccountStatus_GetDisposed(finalStatus)==NULL) {
+	    const AB_VALUE *v=AB_AccountStatus_GetDisposed(bestBooked);
+	    AB_AccountStatus_SetDisposed(finalStatus, v);
+	  }
+	  AB_AccountStatus_SetTime(finalStatus,
+				   AB_AccountStatus_GetTime(bestBooked));
 	}
-	a->setAccountStatus(bestBooked);
+
+	if (bestNoted) {
+	  const AB_BALANCE *bal;
+
+	  bal=AB_AccountStatus_GetNotedBalance(bestNoted);
+	  if (bal)
+	    AB_AccountStatus_SetNotedBalance(finalStatus, bal);
+	  if (AB_AccountStatus_GetBankLine(finalStatus)==NULL) {
+	    const AB_VALUE *v=AB_AccountStatus_GetBankLine(bestNoted);
+	    AB_AccountStatus_SetBankLine(finalStatus, v);
+	  }
+	  if (AB_AccountStatus_GetDisposable(finalStatus)==NULL) {
+	    const AB_VALUE *v=AB_AccountStatus_GetDisposable(bestNoted);
+	    AB_AccountStatus_SetDisposable(finalStatus, v);
+	  }
+	  if (AB_AccountStatus_GetDisposed(finalStatus)==NULL) {
+	    const AB_VALUE *v=AB_AccountStatus_GetDisposed(bestNoted);
+	    AB_AccountStatus_SetDisposed(finalStatus, v);
+	  }
+					 
+	  if (AB_AccountStatus_GetTime(finalStatus)==NULL)
+	    AB_AccountStatus_SetTime(finalStatus,
+				     AB_AccountStatus_GetTime(bestNoted));
+	}
+	a->setAccountStatus(finalStatus);
       }
-      else if (bestNoted) {
-        a->setAccountStatus(bestNoted);
+
+      /* set account name if it is empty in DB */
+      if (a->getAccountName().empty()) {
+	const char *s;
+
+	s=AB_ImExporterAccountInfo_GetAccountName(ai);
+	if (s)
+          a->setAccountName(s);
       }
     }
     ai=AB_ImExporterContext_GetNextAccountInfo(_context);
@@ -1079,10 +1087,14 @@ bool TransactionImporter::importContext(){
 
 TransactionImporter::AccountData*
 TransactionImporter::getAccountData(const char *s1,
-                                    const char *s2) {
+				    const char *s2,
+				    const char *s3,
+				    AB_ACCOUNT_TYPE t) {
   std::list<AccountData*>::iterator it;
   std::string bankCode;
   std::string accountNumber;
+  std::string accountName;
+  std::string alias;
   Account *a;
 
   DBG_DEBUG(0, "Checking for account \"%s/%s\"", s1, s2);
@@ -1091,41 +1103,70 @@ TransactionImporter::getAccountData(const char *s1,
     bankCode=s1;
   if (s2)
     accountNumber=s2;
+  if (s3)
+    accountName=s3;
+
+  alias=bankCode+":"+accountNumber+":"+accountName;
 
   a=0;
   for (it=_accountDataList.begin();
        it!=_accountDataList.end();
        it++) {
-    if ((*it)->bankCode()==bankCode &&
-        (*it)->accountNumber()==accountNumber)
+    if (((*it)->bankCode()==bankCode &&
+	 (*it)->accountNumber()==accountNumber)
+	||
+	(bankCode.empty() &&
+	 accountNumber.empty() &&
+	 !accountName.empty() &&
+	 (*it)->accountName()==accountName))
       return *it;
   } // for
 
+  /* find by alias */
+  for (it=_accountDataList.begin();
+       it!=_accountDataList.end();
+       it++) {
+    std::list<std::string>::const_iterator sit;
+
+    for (sit=(*it)->aliases().begin();
+	 sit!=(*it)->aliases().end();
+	 sit++) {
+      if (*sit==alias)
+	return *it;
+    }
+  } // for
+
   if (!bankCode.empty() && !accountNumber.empty())
     a=_app->findAccount(bankCode.c_str(), accountNumber.c_str());
   else if (bankCode.empty() && !accountNumber.empty())
     a=_app->findAccount("*", accountNumber.c_str());
+  else
+    a=_app->findAccountByName(accountName.c_str());
   if (!a) {
     // TODO: check for aliases
     // TODO: ask the user for alias
-    a=askUserForAccount(bankCode, accountNumber);
+    a=askUserForAccount(bankCode, accountNumber, accountName, t);
     if (!a) {
       DBG_NOTICE(0, "Account not found");
-      GWEN_WaitCallback_Log(GWEN_LoggerLevelNotice, "User aborted");
-      return 0;
+      return NULL;
     }
 
     for (it=_accountDataList.begin();
          it!=_accountDataList.end();
          it++) {
       if ((*it)->bankCode()==a->getBankCode() &&
-          (*it)->accountNumber()==a->getAccountNumber())
-        return *it;
+	  (*it)->accountNumber()==a->getAccountNumber()) {
+	(*it)->addAlias(alias);
+	return *it;
+      }
     } // for
   }
 
   DBG_DEBUG(0, "Adding account data");
-  _accountDataList.push_back(new AccountData(a));
+
+  AccountData *ad=new AccountData(a);
+  ad->addAlias(alias);
+  _accountDataList.push_back(ad);
   return _accountDataList.back();
 }
 
