@@ -1,7 +1,7 @@
 /***************************************************************************
  $RCSfile$
                              -------------------
-    cvs         : $Id: transaction.cpp 396 2006-05-29 17:00:16Z martin $
+    cvs         : $Id: transaction.cpp 438 2008-06-23 17:45:43Z martin $
     begin       : Mon Mar 01 2004
     copyright   : (C) 2004 by Martin Preuss
     email       : martin@libchipcard.de
@@ -19,10 +19,11 @@
 #include "app.h"
 
 #include <stdlib.h>
-#include <gwenhywfar/md.h>
+#include <gwenhywfar/mdigest.h>
 #include <gwenhywfar/buffer.h>
 #include <gwenhywfar/text.h>
 #include <gwenhywfar/debug.h>
+#include <gwenhywfar/types.h>
 #include <qstring.h>
 
 #if !defined(__GNUC__) && defined(WIN32)
@@ -80,6 +81,13 @@ Transaction::Transaction(const AB_TRANSACTION *t)
   p=AB_Transaction_GetLocalBankCode(t);
   if (p)
     _localBankCode=p;
+  p=AB_Transaction_GetLocalIban(t);
+  if (p)
+    _localIban=p;
+  p=AB_Transaction_GetLocalBic(t);
+  if (p)
+    _localBic=p;
+
   p=AB_Transaction_GetLocalAccountNumber(t);
   if (p)
     _localAccountNumber=p;
@@ -108,6 +116,13 @@ Transaction::Transaction(const AB_TRANSACTION *t)
   p=AB_Transaction_GetRemoteSuffix(t);
   if (p)
     _remoteSuffix=p;
+  p=AB_Transaction_GetRemoteIban(t);
+  if (p)
+    _remoteIban=p;
+  p=AB_Transaction_GetRemoteBic(t);
+  if (p)
+    _remoteBic=p;
+
   sl=AB_Transaction_GetRemoteName(t);
   if (sl) {
     GWEN_STRINGLISTENTRY *se;
@@ -193,6 +208,10 @@ AB_TRANSACTION *Transaction::toBankingTransaction() {
     AB_Transaction_SetLocalBankCode(t, _localBankCode.c_str());
   if (!_localAccountNumber.empty())
     AB_Transaction_SetLocalAccountNumber(t, _localAccountNumber.c_str());
+  if (!_localIban.empty())
+    AB_Transaction_SetRemoteIban(t, _localIban.c_str());
+  if (!_localBic.empty())
+    AB_Transaction_SetRemoteBic(t, _localBic.c_str());
   if (!_localSuffix.empty())
     AB_Transaction_SetLocalSuffix(t, _localSuffix.c_str());
   if (!_localOwnerName.empty())
@@ -208,6 +227,10 @@ AB_TRANSACTION *Transaction::toBankingTransaction() {
     AB_Transaction_SetRemoteBankLocation(t, _remoteBankLocation.c_str());
   if (!_remoteAccountNumber.empty())
     AB_Transaction_SetRemoteAccountNumber(t, _remoteAccountNumber.c_str());
+  if (!_remoteIban.empty())
+    AB_Transaction_SetRemoteIban(t, _remoteIban.c_str());
+  if (!_remoteBic.empty())
+    AB_Transaction_SetRemoteBic(t, _remoteBic.c_str());
   if (!_remoteName.empty()) {
     std::list<std::string>::iterator sit;
 
@@ -280,6 +303,8 @@ Transaction::Transaction(const Transaction &t)
   _localCountry=t._localCountry;
   _localBankCode=t._localBankCode;
   _localAccountNumber=t._localAccountNumber;
+  _localIban=t._localIban;
+  _localBic=t._localBic;
   _localSuffix=t._localSuffix;
   _localOwnerName=t._localOwnerName;
 
@@ -289,6 +314,8 @@ Transaction::Transaction(const Transaction &t)
   _remoteBankLocation=t._remoteBankLocation;
   _remoteAccountNumber=t._remoteAccountNumber;
   _remoteSuffix=t._remoteSuffix;
+  _remoteIban=t._remoteIban;
+  _remoteBic=t._remoteBic;
   _remoteName=t._remoteName;
   _category=t._category;
   _payee=t._payee;
@@ -407,6 +434,34 @@ void Transaction::setLocalSuffix(const std::string &s){
 
 
 
+const std::string &Transaction::getLocalIban() const{
+  return _localIban;
+}
+
+
+
+void Transaction::setLocalIban(const std::string &s){
+  _localIban=s;
+  _hash.erase();
+  _modified=true;
+}
+
+
+
+const std::string &Transaction::getLocalBic() const{
+  return _localBic;
+}
+
+
+
+void Transaction::setLocalBic(const std::string &s){
+  _localBic=s;
+  _hash.erase();
+  _modified=true;
+}
+
+
+
 const std::string &Transaction::getLocalName() const{
   return _localOwnerName;
 }
@@ -491,6 +546,20 @@ void Transaction::setRemoteIban(const std::string &s){
 
 
 
+const std::string &Transaction::getRemoteBic() const{
+  return _remoteBic;
+}
+
+
+
+void Transaction::setRemoteBic(const std::string &s){
+  _remoteBic=s;
+  _hash.erase();
+  _modified=true;
+}
+
+
+
 const std::string &Transaction::getRemoteAccountNumber() const{
   return _remoteAccountNumber;
 }
@@ -715,7 +784,8 @@ std::string Transaction::_makeHash() {
   char buffer[32];
   GWEN_BUFFER *dbuf;
   char hbuf[32];
-  unsigned int hsize;
+  GWEN_MDIGEST *md;
+  int rv;
 
   s=_localCountry;
   s+="#";
@@ -746,29 +816,27 @@ std::string Transaction::_makeHash() {
   s+="#";
 
   if (_valutaDate) {
-    snprintf(buffer, sizeof(buffer), GWEN_TYPE_TMPL_UINT32,
+    snprintf(buffer, sizeof(buffer), "%u",
 	     (unsigned int)GWEN_Time_Seconds(_valutaDate));
     s+=buffer;
   }
   s+="#";
   if (_date) {
-    snprintf(buffer, sizeof(buffer), GWEN_TYPE_TMPL_UINT32,
+    snprintf(buffer, sizeof(buffer), "%u",
 	     (unsigned int)GWEN_Time_Seconds(_date));
     s+=buffer;
   }
   s+="#";
   if (_value) {
-    if (AB_Value_IsValid(_value)) {
-      const char *p;
+    const char *p;
 
-      snprintf(buffer, sizeof(buffer), "%lf",
-	       AB_Value_GetValue(_value));
-      s+=buffer;
-      s+=" ";
-      p=AB_Value_GetCurrency(_value);
-      if (p)
-        s+=p;
-    }
+    snprintf(buffer, sizeof(buffer), "%lf",
+	     AB_Value_GetValueAsDouble(_value));
+    s+=buffer;
+    s+=" ";
+    p=AB_Value_GetCurrency(_value);
+    if (p)
+      s+=p;
   }
   s+="#";
   s+=_transactionKey;
@@ -798,18 +866,35 @@ std::string Transaction::_makeHash() {
   GWEN_Text_CondenseBuffer(dbuf);
 
   /* now hash the data */
-  hsize=sizeof(hbuf);
-  if (GWEN_MD_Hash("rmd160",
-                   GWEN_Buffer_GetStart(dbuf),
-                   GWEN_Buffer_GetUsedBytes(dbuf),
-                   hbuf,
-                   &hsize)) {
-    DBG_ERROR(0, "Could not hash transaction data");
+  md=GWEN_MDigest_Rmd160_new();
+  assert(md);
+  rv=GWEN_MDigest_Begin(md);
+  if (rv<0) {
+    DBG_ERROR(0, "here (%d)", rv);
+    GWEN_MDigest_free(md);
     GWEN_Buffer_free(dbuf);
     return "";
   }
-  GWEN_Buffer_Reset(dbuf);
-  if (GWEN_Text_ToHexBuffer(hbuf, hsize, dbuf, 0, 0, 0)) {
+  rv=GWEN_MDigest_Update(md,
+			 (const uint8_t*)GWEN_Buffer_GetStart(dbuf),
+			 GWEN_Buffer_GetUsedBytes(dbuf));
+  if (rv<0) {
+    DBG_ERROR(0, "here (%d)", rv);
+    GWEN_MDigest_free(md);
+    GWEN_Buffer_free(dbuf);
+    return "";
+  }
+  rv=GWEN_MDigest_End(md);
+  if (rv<0) {
+    DBG_ERROR(0, "here (%d)", rv);
+    GWEN_MDigest_free(md);
+    GWEN_Buffer_free(dbuf);
+    return "";
+  }
+  memmove(hbuf, GWEN_MDigest_GetDigestPtr(md), 20);
+  GWEN_MDigest_free(md);
+
+  if (GWEN_Text_ToHexBuffer(hbuf, 20, dbuf, 0, 0, 0)) {
     DBG_ERROR(0, "Error converting to hex");
     GWEN_Buffer_free(dbuf);
     return "";
@@ -923,6 +1008,12 @@ bool Transaction::toDb(GWEN_DB_NODE *db) const {
   if (!_localSuffix.empty())
     GWEN_DB_SetCharValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS,
                          "localSuffix", _localSuffix.c_str());
+  if (!_localIban.empty())
+    GWEN_DB_SetCharValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS,
+                         "localIban", _localIban.c_str());
+  if (!_localBic.empty())
+    GWEN_DB_SetCharValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS,
+                         "localBic", _localBic.c_str());
 
   if (!_remoteCountry.empty())
     GWEN_DB_SetCharValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS,
@@ -936,6 +1027,9 @@ bool Transaction::toDb(GWEN_DB_NODE *db) const {
   if (!_remoteIban.empty())
     GWEN_DB_SetCharValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS,
                          "remoteIban", _remoteIban.c_str());
+  if (!_remoteBic.empty())
+    GWEN_DB_SetCharValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS,
+                         "remoteBic", _remoteBic.c_str());
   if (!_remoteBankName.empty())
     GWEN_DB_SetCharValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS,
                          "remoteBankName", _remoteBankName.c_str());
@@ -1039,12 +1133,15 @@ bool Transaction::fromDb(GWEN_DB_NODE *db) {
   _localCountry=GWEN_DB_GetCharValue(db, "localCountry", 0, "");
   _localBankCode=GWEN_DB_GetCharValue(db, "localBankCode", 0, "");
   _localAccountNumber=GWEN_DB_GetCharValue(db, "localAccountNumber", 0, "");
+  _localIban=GWEN_DB_GetCharValue(db, "localIban", 0, "");
+  _localBic=GWEN_DB_GetCharValue(db, "localBic", 0, "");
 
   _remoteCountry=GWEN_DB_GetCharValue(db, "remoteCountry", 0, "");
   _remoteBankCode=GWEN_DB_GetCharValue(db, "remoteBankCode", 0, "");
   _remoteBankName=GWEN_DB_GetCharValue(db, "remoteBankName", 0, "");
   _remoteBankLocation=GWEN_DB_GetCharValue(db, "remoteBankLocation", 0, "");
   _remoteIban=GWEN_DB_GetCharValue(db, "remoteIban", 0, "");
+  _remoteBic=GWEN_DB_GetCharValue(db, "remoteBic", 0, "");
   _remoteAccountNumber=GWEN_DB_GetCharValue(db, "remoteAccountNumber",
                                             0, "");
   _category=GWEN_DB_GetCharValue(db, "category", 0, "");
@@ -1100,6 +1197,8 @@ void Transaction::addPurpose(const std::string &s){
   _modified=true;
 }
 
+
+
 std::string Transaction::toHtmlInfo() const {
   std::string tmp;
   const std::list<std::string> sl;
@@ -1170,7 +1269,7 @@ std::string Transaction::toHtmlInfo() const {
     char numbuf[32];
     const char *s;
 
-    if (AB_Value_GetValue(v)>0)
+    if (AB_Value_GetValueAsDouble(v)>0)
       tmp+="<font color=\"dark green\">";
     else
       tmp+="<font color=red>";
@@ -1179,7 +1278,8 @@ std::string Transaction::toHtmlInfo() const {
     // This is MSVC compiler which doesnt have snprintf()
     sprintf(numbuf, "%.2lf", AB_Value_GetValue(v));
 #else
-    snprintf(numbuf, sizeof(numbuf), "%.2lf", AB_Value_GetValue(v));
+    snprintf(numbuf, sizeof(numbuf), "%.2lf",
+	     AB_Value_GetValueAsDouble(v));
 #endif
     tmp+=numbuf;
     s=AB_Value_GetCurrency(v);
@@ -1199,13 +1299,13 @@ std::string Transaction::toHtmlInfo() const {
 
 
 
-GWEN_TYPE_UINT32 Transaction::getTransactionId() const{
+uint32_t Transaction::getTransactionId() const{
   return _transactionId;
 }
 
 
 
-void Transaction::setTransactionId(GWEN_TYPE_UINT32 id){
+void Transaction::setTransactionId(uint32_t id){
   _transactionId=id;
   _modified=true;
 }
@@ -1252,7 +1352,7 @@ bool Transaction::matchFuzzy(const Transaction &t){
 
   match=true;
   if (v1 && v2) {
-    if (AB_Value_GetValue(v1)!=AB_Value_GetValue(v2))
+    if (AB_Value_Compare(v1, v2)!=0)
       match=false;
   }
 
@@ -1272,7 +1372,7 @@ bool Transaction::matchFuzzy(const Transaction &t){
 
 
 bool Transaction::matchFuzzy(const Transaction &t,
-                             GWEN_TYPE_UINT32 threshold){
+                             uint32_t threshold){
   const AB_VALUE *v1, *v2;
   std::string b1, b2;
   std::string a1, a2;
@@ -1287,7 +1387,7 @@ bool Transaction::matchFuzzy(const Transaction &t,
 
   match=true;
   if (v1 && v2) {
-    if (AB_Value_GetValue(v1)!=AB_Value_GetValue(v2))
+    if (AB_Value_Compare(v1, v2)!=0)
       match=false;
   }
 
@@ -1304,7 +1404,7 @@ bool Transaction::matchFuzzy(const Transaction &t,
   if (match) {
     std::string p1, p2;
     std::list<std::string>::const_iterator sit;
-    GWEN_TYPE_UINT32 fm;
+    uint32_t fm;
 
     for (sit=_purpose.begin();
          sit!=_purpose.end();
@@ -1315,7 +1415,7 @@ bool Transaction::matchFuzzy(const Transaction &t,
          sit++)
       p2+=*sit;
     fm=App_FuzzyCompare(p1.c_str(), p2.c_str());
-    DBG_ERROR(0, "Fuzzy match: "GWEN_TYPE_TMPL_UINT32, fm);
+    DBG_ERROR(0, "Fuzzy match: %u", fm);
     if (fm<threshold) {
       match=false;
     }
@@ -1411,29 +1511,27 @@ std::string Transaction::getHashData() {
   s+="#";
 
   if (_valutaDate) {
-    snprintf(buffer, sizeof(buffer), GWEN_TYPE_TMPL_UINT32,
+    snprintf(buffer, sizeof(buffer), "%u",
 	     (unsigned int)GWEN_Time_Seconds(_valutaDate));
     s+=buffer;
   }
   s+="#";
   if (_date) {
-    snprintf(buffer, sizeof(buffer), GWEN_TYPE_TMPL_UINT32,
+    snprintf(buffer, sizeof(buffer), "%u",
 	     (unsigned int)GWEN_Time_Seconds(_date));
     s+=buffer;
   }
   s+="#";
   if (_value) {
-    if (AB_Value_IsValid(_value)) {
-      const char *p;
+    const char *p;
 
-      snprintf(buffer, sizeof(buffer), "%lf",
-	       AB_Value_GetValue(_value));
-      s+=buffer;
-      s+=" ";
-      p=AB_Value_GetCurrency(_value);
-      if (p)
-        s+=p;
-    }
+    snprintf(buffer, sizeof(buffer), "%lf",
+	     AB_Value_GetValueAsDouble(_value));
+    s+=buffer;
+    s+=" ";
+    p=AB_Value_GetCurrency(_value);
+    if (p)
+      s+=p;
   }
   s+="#";
   s+=_transactionKey;
